@@ -1,17 +1,17 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
-
-from tryton.gui.window.view_form.screen import Screen
-from .widget import Widget
-from tryton.gui.window.win_search import WinSearch
-from tryton.gui.window.win_form import WinForm
-import tryton.common as common
 import gettext
-from tryton.common.placeholder_entry import PlaceholderEntry
+
+from gi.repository import Gdk, Gtk
+
+import tryton.common as common
+from .widget import Widget
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.domain_parser import quote
-from tryton.common.widget_style import widget_class
+from tryton.common.underline import set_underline
+from tryton.gui.window.view_form.screen import Screen
+from tryton.gui.window.win_search import WinSearch
+from tryton.gui.window.win_form import WinForm
 
 _ = gettext.gettext
 
@@ -22,32 +22,33 @@ class Many2Many(Widget):
     def __init__(self, view, attrs):
         super(Many2Many, self).__init__(view, attrs)
 
-        self.widget = gtk.Frame()
-        self.widget.set_shadow_type(gtk.SHADOW_NONE)
+        self.widget = Gtk.Frame()
+        self.widget.set_shadow_type(Gtk.ShadowType.NONE)
         self.widget.get_accessible().set_name(attrs.get('string', ''))
-        vbox = gtk.VBox(homogeneous=False, spacing=5)
+        vbox = Gtk.VBox(homogeneous=False, spacing=5)
         self.widget.add(vbox)
         self._readonly = True
         self._required = False
         self._position = 0
 
-        hbox = gtk.HBox(homogeneous=False, spacing=0)
+        hbox = Gtk.HBox(homogeneous=False, spacing=0)
         hbox.set_border_width(2)
 
-        self.title = gtk.Label(attrs.get('string', ''))
-        self.title.set_alignment(0.0, 0.5)
-        hbox.pack_start(self.title, expand=True, fill=True)
+        self.title = Gtk.Label(
+            label=set_underline(attrs.get('string', '')),
+            use_underline=True, halign=Gtk.Align.START)
+        hbox.pack_start(self.title, expand=True, fill=True, padding=0)
 
-        hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+        hbox.pack_start(Gtk.VSeparator(), expand=False, fill=True, padding=0)
 
         tooltips = common.Tooltips()
 
-        self.wid_text = PlaceholderEntry()
+        self.wid_text = Gtk.Entry()
         self.wid_text.set_placeholder_text(_('Search'))
         self.wid_text.set_property('width_chars', 13)
         self.wid_text.connect('focus-out-event', self._focus_out)
         self.focus_out = True
-        hbox.pack_start(self.wid_text, expand=True, fill=True)
+        hbox.pack_start(self.wid_text, expand=True, fill=True, padding=0)
 
         if int(self.attrs.get('completion', 1)):
             self.wid_completion = get_completion(
@@ -62,36 +63,28 @@ class Many2Many(Widget):
         else:
             self.wid_completion = None
 
-        self.but_add = gtk.Button()
+        self.but_add = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_add, _('Add existing record'))
         self.but_add.connect('clicked', self._sig_add)
-        img_add = gtk.Image()
-        img_add.set_from_stock('tryton-list-add',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_add.set_alignment(0.5, 0.5)
-        self.but_add.add(img_add)
-        self.but_add.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_add, expand=False, fill=False)
+        self.but_add.add(common.IconFactory.get_image(
+                'tryton-add', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_add.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_add, expand=False, fill=False, padding=0)
 
-        self.but_remove = gtk.Button()
-        tooltips.set_tip(self.but_remove, _('Remove selected record <Del>'))
+        self.but_remove = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_remove, _('Remove selected record'))
         self.but_remove.connect('clicked', self._sig_remove)
-        img_remove = gtk.Image()
-        img_remove.set_from_stock('tryton-list-remove',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_remove.set_alignment(0.5, 0.5)
-        self.but_remove.add(img_remove)
-        self.but_remove.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_remove, expand=False, fill=False)
-
-        hbox.set_focus_chain([self.wid_text])
+        self.but_remove.add(common.IconFactory.get_image(
+                'tryton-remove', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_remove.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_remove, expand=False, fill=False, padding=0)
 
         tooltips.enable()
 
-        frame = gtk.Frame()
+        frame = Gtk.Frame()
         frame.add(hbox)
-        frame.set_shadow_type(gtk.SHADOW_OUT)
-        vbox.pack_start(frame, expand=False, fill=True)
+        frame.set_shadow_type(Gtk.ShadowType.OUT)
+        vbox.pack_start(frame, expand=False, fill=True, padding=0)
 
         self.screen = Screen(attrs['relation'],
             view_ids=attrs.get('view_ids', '').split(','),
@@ -100,32 +93,35 @@ class Many2Many(Widget):
             limit=None)
         self.screen.signal_connect(self, 'record-message', self._sig_label)
 
-        vbox.pack_start(self.screen.widget, expand=True, fill=True)
+        vbox.pack_start(self.screen.widget, expand=True, fill=True, padding=0)
+
+        self.title.set_mnemonic_widget(
+            self.screen.current_view.mnemonic_widget)
 
         self.screen.widget.connect('key_press_event', self.on_keypress)
         self.wid_text.connect('key_press_event', self.on_keypress)
 
     def on_keypress(self, widget, event):
         editable = self.wid_text.get_editable()
-        activate_keys = [gtk.keysyms.Tab, gtk.keysyms.ISO_Left_Tab]
-        remove_keys = [gtk.keysyms.Delete, gtk.keysyms.KP_Delete]
+        activate_keys = [Gdk.KEY_Tab, Gdk.KEY_ISO_Left_Tab]
+        remove_keys = [Gdk.KEY_Delete, Gdk.KEY_KP_Delete]
         if not self.wid_completion:
-            activate_keys.append(gtk.keysyms.Return)
+            activate_keys.append(Gdk.KEY_Return)
         if widget == self.screen.widget:
-            if event.keyval == gtk.keysyms.F3 and editable:
+            if event.keyval == Gdk.KEY_F3 and editable:
                 self._sig_add()
                 return True
-            elif event.keyval == gtk.keysyms.F2:
+            elif event.keyval == Gdk.KEY_F2:
                 self._sig_edit()
                 return True
             elif event.keyval in remove_keys and editable:
                 self._sig_remove()
                 return True
         elif widget == self.wid_text:
-            if event.keyval == gtk.keysyms.F3:
+            if event.keyval == Gdk.KEY_F3:
                 self._sig_new()
                 return True
-            elif event.keyval == gtk.keysyms.F2:
+            elif event.keyval == Gdk.KEY_F2:
                 self._sig_add()
                 return True
             elif event.keyval in activate_keys and self.wid_text.get_text():
@@ -144,8 +140,9 @@ class Many2Many(Widget):
         add_remove = self.record.expr_eval(self.attrs.get('add_remove'))
         if add_remove:
             domain = [domain, add_remove]
-        context = self.field.get_context(self.record)
-        value = self.wid_text.get_text().decode('utf-8')
+        context = self.field.get_search_context(self.record)
+        order = self.field.get_search_order(self.record)
+        value = self.wid_text.get_text()
 
         self.focus_out = False
 
@@ -158,13 +155,16 @@ class Many2Many(Widget):
             self.screen.set_cursor()
             self.wid_text.set_text('')
         win = WinSearch(self.attrs['relation'], callback, sel_multi=True,
-            context=context, domain=domain,
+            context=context, domain=domain, order=order,
             view_ids=self.attrs.get('view_ids', '').split(','),
             views_preload=self.attrs.get('views', {}),
             new=self.attrs.get('create', True),
             title=self.attrs.get('string'))
         win.screen.search_filter(quote(value))
-        win.show()
+        if len(win.screen.group) == 1:
+            win.response(None, Gtk.ResponseType.OK)
+        else:
+            win.show()
 
     def _sig_remove(self, *args):
         self.screen.remove(remove=True)
@@ -229,10 +229,8 @@ class Many2Many(Widget):
         self._set_label_state()
 
     def _set_label_state(self):
-        attrlist = common.get_label_attributes(self._readonly, self._required)
-        self.title.set_attributes(attrlist)
-        widget_class(self.title, 'readonly', self._readonly)
-        widget_class(self.title, 'required', self._required)
+        common.apply_label_attributes(
+            self.title, self._readonly, self._required)
 
     def _set_button_sensitive(self):
         if self.record and self.field:
@@ -254,21 +252,21 @@ class Many2Many(Widget):
         self._position = signal_data[0]
         self._set_button_sensitive()
 
-    def display(self, record, field):
-        super(Many2Many, self).display(record, field)
-        if field is None:
+    def display(self):
+        super(Many2Many, self).display()
+        if not self.field:
             self.screen.new_group()
             self.screen.current_record = None
             self.screen.parent = None
             self.screen.display()
             return False
-        new_group = field.get_client(record)
+        new_group = self.field.get_client(self.record)
         if id(self.screen.group) != id(new_group):
             self.screen.group = new_group
         self.screen.display()
         return True
 
-    def set_value(self, record, field):
+    def set_value(self):
         self.screen.current_view.set_value()
         return True
 
