@@ -1,18 +1,18 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
 import gettext
 import itertools
 
+from gi.repository import Gdk, Gtk
+
 from .widget import Widget
 from tryton.gui.window.view_form.screen import Screen
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
 import tryton.common as common
-from tryton.common.placeholder_entry import PlaceholderEntry
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.domain_parser import quote
-from tryton.common.widget_style import widget_class
+from tryton.common.underline import set_underline
 
 _ = gettext.gettext
 
@@ -23,73 +23,66 @@ class One2Many(Widget):
     def __init__(self, view, attrs):
         super(One2Many, self).__init__(view, attrs)
 
-        self.widget = gtk.Frame()
-        self.widget.set_shadow_type(gtk.SHADOW_NONE)
+        self.widget = Gtk.Frame()
+        self.widget.set_shadow_type(Gtk.ShadowType.NONE)
         self.widget.get_accessible().set_name(attrs.get('string', ''))
-        vbox = gtk.VBox(homogeneous=False, spacing=2)
+        vbox = Gtk.VBox(homogeneous=False, spacing=2)
         self.widget.add(vbox)
         self._readonly = True
         self._required = False
         self._position = 0
         self._length = 0
 
-        self.title_box = hbox = gtk.HBox(homogeneous=False, spacing=0)
+        self.title_box = hbox = Gtk.HBox(homogeneous=False, spacing=0)
         hbox.set_border_width(2)
 
-        self.title = gtk.Label(attrs.get('string', ''))
-        self.title.set_alignment(0.0, 0.5)
-        hbox.pack_start(self.title, expand=True, fill=True)
+        self.title = Gtk.Label(
+            label=set_underline(attrs.get('string', '')),
+            use_underline=True, halign=Gtk.Align.START)
+        hbox.pack_start(self.title, expand=True, fill=True, padding=0)
 
-        hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+        hbox.pack_start(Gtk.VSeparator(), expand=False, fill=True, padding=0)
 
         tooltips = common.Tooltips()
 
-        but_switch = gtk.Button()
+        but_switch = Gtk.Button(can_focus=False)
         tooltips.set_tip(but_switch, _('Switch'))
         but_switch.connect('clicked', self.switch_view)
-        img_switch = gtk.Image()
-        img_switch.set_from_stock('tryton-fullscreen',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_switch.set_alignment(0.5, 0.5)
-        but_switch.add(img_switch)
-        but_switch.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(but_switch, expand=False, fill=False)
+        but_switch.add(common.IconFactory.get_image(
+                'tryton-switch', Gtk.IconSize.SMALL_TOOLBAR))
+        but_switch.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(but_switch, expand=False, fill=False, padding=0)
 
-        self.but_pre = gtk.Button()
+        self.but_pre = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_pre, _('Previous'))
         self.but_pre.connect('clicked', self._sig_previous)
-        img_pre = gtk.Image()
-        img_pre.set_from_stock('tryton-go-previous',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_pre.set_alignment(0.5, 0.5)
-        self.but_pre.add(img_pre)
-        self.but_pre.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_pre, expand=False, fill=False)
+        self.but_pre.add(common.IconFactory.get_image(
+                'tryton-back', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_pre.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_pre, expand=False, fill=False, padding=0)
 
-        self.label = gtk.Label('(0,0)')
-        hbox.pack_start(self.label, expand=False, fill=False)
+        self.label = Gtk.Label(label='(0,0)')
+        hbox.pack_start(self.label, expand=False, fill=False, padding=0)
 
-        self.but_next = gtk.Button()
+        self.but_next = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_next, _('Next'))
         self.but_next.connect('clicked', self._sig_next)
-        img_next = gtk.Image()
-        img_next.set_from_stock('tryton-go-next', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_next.set_alignment(0.5, 0.5)
-        self.but_next.add(img_next)
-        self.but_next.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_next, expand=False, fill=False)
+        self.but_next.add(common.IconFactory.get_image(
+                'tryton-forward', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_next.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_next, expand=False, fill=False, padding=0)
 
-        hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+        hbox.pack_start(Gtk.VSeparator(), expand=False, fill=True, padding=0)
 
         self.focus_out = True
         self.wid_completion = None
         if attrs.get('add_remove'):
 
-            self.wid_text = PlaceholderEntry()
+            self.wid_text = Gtk.Entry()
             self.wid_text.set_placeholder_text(_('Search'))
             self.wid_text.set_property('width_chars', 13)
             self.wid_text.connect('focus-out-event', self._focus_out)
-            hbox.pack_start(self.wid_text, expand=True, fill=True)
+            hbox.pack_start(self.wid_text, expand=True, fill=True, padding=0)
 
             if int(self.attrs.get('completion', 1)):
                 access = common.MODELACCESS[attrs['relation']]
@@ -103,82 +96,65 @@ class One2Many(Widget):
                 self.wid_text.set_completion(self.wid_completion)
                 self.wid_text.connect('changed', self._update_completion)
 
-            self.but_add = gtk.Button()
+            self.but_add = Gtk.Button(can_focus=False)
             tooltips.set_tip(self.but_add, _('Add existing record'))
             self.but_add.connect('clicked', self._sig_add)
-            img_add = gtk.Image()
-            img_add.set_from_stock('tryton-list-add',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_add.set_alignment(0.5, 0.5)
-            self.but_add.add(img_add)
-            self.but_add.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(self.but_add, expand=False, fill=False)
+            self.but_add.add(common.IconFactory.get_image(
+                    'tryton-add', Gtk.IconSize.SMALL_TOOLBAR))
+            self.but_add.set_relief(Gtk.ReliefStyle.NONE)
+            hbox.pack_start(self.but_add, expand=False, fill=False, padding=0)
 
-            self.but_remove = gtk.Button()
+            self.but_remove = Gtk.Button(can_focus=False)
             tooltips.set_tip(self.but_remove,
                 _('Remove selected record'))
             self.but_remove.connect('clicked', self._sig_remove, True)
-            img_remove = gtk.Image()
-            img_remove.set_from_stock('tryton-list-remove',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_remove.set_alignment(0.5, 0.5)
-            self.but_remove.add(img_remove)
-            self.but_remove.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(self.but_remove, expand=False, fill=False)
+            self.but_remove.add(common.IconFactory.get_image(
+                    'tryton-remove', Gtk.IconSize.SMALL_TOOLBAR))
+            self.but_remove.set_relief(Gtk.ReliefStyle.NONE)
+            hbox.pack_start(
+                self.but_remove, expand=False, fill=False, padding=0)
 
-            hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+            hbox.pack_start(
+                Gtk.VSeparator(), expand=False, fill=True, padding=0)
 
-        self.but_new = gtk.Button()
-        tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
+        self.but_new = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_new, _('Create a new record'))
         self.but_new.connect('clicked', self._sig_new)
-        img_new = gtk.Image()
-        img_new.set_from_stock('tryton-new', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_new.set_alignment(0.5, 0.5)
-        self.but_new.add(img_new)
-        self.but_new.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_new, expand=False, fill=False)
+        self.but_new.add(common.IconFactory.get_image(
+                'tryton-create', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_new.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_new, expand=False, fill=False, padding=0)
 
-        self.but_open = gtk.Button()
-        tooltips.set_tip(self.but_open, _('Edit selected record <F2>'))
+        self.but_open = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_open, _('Edit selected record'))
         self.but_open.connect('clicked', self._sig_edit)
-        img_open = gtk.Image()
-        img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_open.set_alignment(0.5, 0.5)
-        self.but_open.add(img_open)
-        self.but_open.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_open, expand=False, fill=False)
+        self.but_open.add(common.IconFactory.get_image(
+                'tryton-open', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_open.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_open, expand=False, fill=False, padding=0)
 
-        self.but_del = gtk.Button()
-        tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
+        self.but_del = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_del, _('Delete selected record'))
         self.but_del.connect('clicked', self._sig_remove, False)
-        img_del = gtk.Image()
-        img_del.set_from_stock('tryton-delete', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_del.set_alignment(0.5, 0.5)
-        self.but_del.add(img_del)
-        self.but_del.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_del, expand=False, fill=False)
+        self.but_del.add(common.IconFactory.get_image(
+                'tryton-delete', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_del.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_del, expand=False, fill=False, padding=0)
 
-        self.but_undel = gtk.Button()
+        self.but_undel = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_undel, _('Undelete selected record <Ins>'))
         self.but_undel.connect('clicked', self._sig_undelete)
-        img_undel = gtk.Image()
-        img_undel.set_from_stock('tryton-undo', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_undel.set_alignment(0.5, 0.5)
-        self.but_undel.add(img_undel)
-        self.but_undel.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_undel, expand=False, fill=False)
-
-        if attrs.get('add_remove'):
-            hbox.set_focus_chain([self.wid_text])
-        else:
-            hbox.set_focus_chain([])
+        self.but_undel.add(common.IconFactory.get_image(
+                'tryton-undo', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_undel.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_undel, expand=False, fill=False, padding=0)
 
         tooltips.enable()
 
-        frame = gtk.Frame()
+        frame = Gtk.Frame()
         frame.add(hbox)
-        frame.set_shadow_type(gtk.SHADOW_OUT)
-        vbox.pack_start(frame, expand=False, fill=True)
+        frame.set_shadow_type(Gtk.ShadowType.OUT)
+        vbox.pack_start(frame, expand=False, fill=True, padding=0)
 
         self.screen = Screen(attrs['relation'],
             mode=attrs.get('mode', 'tree,form').split(','),
@@ -190,7 +166,10 @@ class One2Many(Widget):
         self.screen.pre_validate = bool(int(attrs.get('pre_validate', 0)))
         self.screen.signal_connect(self, 'record-message', self._sig_label)
 
-        vbox.pack_start(self.screen.widget, expand=True, fill=True)
+        vbox.pack_start(self.screen.widget, expand=True, fill=True, padding=0)
+
+        self.title.set_mnemonic_widget(
+            self.screen.current_view.mnemonic_widget)
 
         self.screen.widget.connect('key_press_event', self.on_keypress)
         if self.attrs.get('add_remove'):
@@ -199,27 +178,37 @@ class One2Many(Widget):
         but_switch.props.sensitive = self.screen.number_of_views > 1
 
     def on_keypress(self, widget, event):
-        if (event.keyval == gtk.keysyms.F3) \
-                and self.but_new.get_property('sensitive'):
+        if ((event.keyval == Gdk.KEY_F3)
+                and self.but_new.get_property('sensitive')):
             self._sig_new(widget)
             return True
-        if event.keyval == gtk.keysyms.F2 \
-                and widget == self.screen.widget:
-            self._sig_edit(widget)
-            return True
-        if (event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete)
-                and widget == self.screen.widget
-                and self.but_del.get_property('sensitive')):
-            self._sig_remove(widget)
-            return True
-        if event.keyval == gtk.keysyms.Insert and widget == self.screen.widget:
+        if event.keyval == Gdk.KEY_F2:
+            if widget == self.screen.widget:
+                self._sig_edit(widget)
+                return True
+            elif widget == self.wid_text:
+                self._sig_add(widget)
+                return True
+        if event.keyval == Gdk.KEY_F4:
+            self.switch_view(widget)
+        if (event.keyval in [Gdk.KEY_Delete, Gdk.KEY_KP_Delete]
+                and widget == self.screen.widget):
+            remove = not (event.state & Gdk.ModifierType.CONTROL_MASK)
+            if remove and self.attrs.get('add_remove'):
+                but = self.but_remove
+            else:
+                but = self.but_del
+            if but.get_property('sensitive'):
+                self._sig_remove(widget, remove)
+                return True
+        if event.keyval == Gdk.KEY_Insert and widget == self.screen.widget:
             self._sig_undelete(widget)
             return True
         if self.attrs.get('add_remove'):
             editable = self.wid_text.get_editable()
-            activate_keys = [gtk.keysyms.Tab, gtk.keysyms.ISO_Left_Tab]
+            activate_keys = [Gdk.KEY_Tab, Gdk.KEY_ISO_Left_Tab]
             if not self.wid_completion:
-                activate_keys.append(gtk.keysyms.Return)
+                activate_keys.append(Gdk.KEY_Return)
             if (widget == self.wid_text
                     and event.keyval in activate_keys
                     and editable
@@ -238,6 +227,12 @@ class One2Many(Widget):
 
     def switch_view(self, widget):
         self.screen.switch_view()
+        mnemonic_widget = self.screen.current_view.mnemonic_widget
+        string = self.attrs.get('string', '')
+        if mnemonic_widget:
+            string = set_underline(string)
+        self.title.set_mnemonic_widget(mnemonic_widget)
+        self.title.set_label(string)
 
     @property
     def modified(self):
@@ -253,10 +248,8 @@ class One2Many(Widget):
         self._set_label_state()
 
     def _set_label_state(self):
-        attrlist = common.get_label_attributes(self._readonly, self._required)
-        self.title.set_attributes(attrlist)
-        widget_class(self.title, 'readonly', self._readonly)
-        widget_class(self.title, 'required', self._required)
+        common.apply_label_attributes(
+            self.title, self._readonly, self._required)
 
     def _set_button_sensitive(self):
         access = common.MODELACCESS[self.screen.model_name]
@@ -269,6 +262,11 @@ class One2Many(Widget):
             o2m_size = None
             size_limit = False
 
+        first = last = False
+        if isinstance(self._position, int):
+            first = self._position <= 1
+            last = self._position >= self._length
+
         self.but_new.set_sensitive(bool(
                 not self._readonly
                 and self.attrs.get('create', True)
@@ -288,10 +286,10 @@ class One2Many(Widget):
                 and access['read']))
         self.but_next.set_sensitive(bool(
                 self._position
-                and self._position < self._length))
+                and not last))
         self.but_pre.set_sensitive(bool(
                 self._position
-                and self._position > 1))
+                and not first))
         if self.attrs.get('add_remove'):
             self.but_add.set_sensitive(bool(
                     not self._readonly
@@ -306,17 +304,6 @@ class One2Many(Widget):
             self.wid_text.set_sensitive(self.but_add.get_sensitive())
             self.wid_text.set_editable(self.but_add.get_sensitive())
 
-        # New button must be added to focus chain to allow keyboard only
-        # creation when there is no existing record on form view.
-        focus_chain = self.title_box.get_focus_chain() or []
-        if o2m_size == 0 and self.screen.current_view.view_type == 'form':
-            if self.but_new not in focus_chain:
-                focus_chain.append(self.but_new)
-        else:
-            if self.but_new in focus_chain:
-                focus_chain.remove(self.but_new)
-        self.title_box.set_focus_chain(focus_chain)
-
     def _validate(self):
         self.view.set_value()
         record = self.screen.current_record
@@ -383,14 +370,16 @@ class One2Many(Widget):
                 search_set()
 
             domain = field.domain_get(first)
-            context = field.get_context(first)
+            context = field.get_search_context(first)
+            order = field.get_search_order(first)
 
             def callback(result):
                 if result:
                     product[field.name] = result
 
             win_search = WinSearch(relation, callback, sel_multi=True,
-                context=context, domain=domain, title=self.attrs.get('string'))
+                context=context, domain=domain, order=order,
+                title=self.attrs.get('string'))
             win_search.win.connect('destroy', search_set)
             win_search.screen.search_filter()
             win_search.show()
@@ -400,8 +389,8 @@ class One2Many(Widget):
             if not product:
                 return
 
-            fields = product.keys()
-            for values in itertools.product(*product.values()):
+            fields = list(product.keys())
+            for values in itertools.product(*list(product.values())):
                 record = self.screen.new(default=False)
                 default_value = default.copy()
                 for field, value in zip(fields, values):
@@ -457,11 +446,11 @@ class One2Many(Widget):
             return
         self.view.set_value()
         domain = self.field.domain_get(self.record)
-        context = self.field.get_context(self.record)
+        context = self.field.get_search_context(self.record)
         domain = [domain, self.record.expr_eval(self.attrs.get('add_remove'))]
         removed_ids = self.field.get_removed_ids(self.record)
         domain = ['OR', domain, ('id', 'in', removed_ids)]
-        text = self.wid_text.get_text().decode('utf-8')
+        text = self.wid_text.get_text()
 
         self.focus_out = False
 
@@ -478,8 +467,9 @@ class One2Many(Widget):
             self.screen.set_cursor()
             self.wid_text.set_text('')
 
+        order = self.field.get_search_order(self.record)
         win = WinSearch(self.attrs['relation'], callback, sel_multi=True,
-            context=context, domain=domain,
+            context=context, domain=domain, order=order,
             view_ids=self.attrs.get('view_ids', '').split(','),
             views_preload=self.attrs.get('views', {}),
             new=self.but_new.get_property('sensitive'),
@@ -490,7 +480,7 @@ class One2Many(Widget):
     def _sig_label(self, screen, signal_data):
         self._position = signal_data[0]
         self._length = signal_data[1]
-        if self._position >= 1:
+        if self._position:
             name = str(self._position)
         else:
             name = '_'
@@ -498,18 +488,18 @@ class One2Many(Widget):
         self.label.set_text(line)
         self._set_button_sensitive()
 
-    def display(self, record, field):
-        super(One2Many, self).display(record, field)
+    def display(self):
+        super(One2Many, self).display()
 
         self._set_button_sensitive()
 
-        if field is None:
+        if not self.field:
             self.screen.new_group()
             self.screen.current_record = None
             self.screen.parent = None
             self.screen.display()
             return False
-        new_group = field.get_client(record)
+        new_group = self.field.get_client(self.record)
 
         if id(self.screen.group) != id(new_group):
             self.screen.group = new_group
@@ -518,9 +508,9 @@ class One2Many(Widget):
                 self.screen.current_record = None
         domain = []
         size_limit = None
-        if record:
-            domain = field.domain_get(record)
-            size_limit = record.expr_eval(self.attrs.get('size'))
+        if self.record:
+            domain = self.field.domain_get(self.record)
+            size_limit = self.record.expr_eval(self.attrs.get('size'))
         if self._readonly:
             if size_limit is None:
                 size_limit = len(self.screen.group)
@@ -532,11 +522,11 @@ class One2Many(Widget):
         self.screen.display()
         return True
 
-    def set_value(self, record, field):
+    def set_value(self):
         self.screen.current_view.set_value()
         if self.screen.modified():  # TODO check if required
-            record.modified_fields.setdefault(field.name)
-            record.signal('record-modified')
+            self.record.modified_fields.setdefault(self.field.name)
+            self.record.signal('record-modified')
         return True
 
     def _completion_match_selected(self, completion, model, iter_):
